#!/usr/bin/env python
#
# Copyright (c) 2015 Serguei Glavatski ( verser  from cnc-club.ru )
# Copyright (c) 2020 Probe Screen NG Developers
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; If not, see <http://www.gnu.org/licenses/>.

import math
import time

import hal

from .base import ProbeScreenBase


class ProbeScreenRotation(ProbeScreenBase):
    # --------------------------
    #
    #  INIT
    #
    # --------------------------
    def __init__(self, halcomp, builder, useropts):
        super(ProbeScreenRotation, self).__init__(halcomp, builder, useropts)

        self.hal_led_auto_rott = self.builder.get_object("hal_led_auto_rott")
        self.chk_auto_rott = self.builder.get_object("chk_auto_rott")
        self.spbtn_offs_angle = self.builder.get_object("spbtn_offs_angle")

        self.chk_auto_rott.set_active(self.prefs.getpref("chk_auto_rott", False, bool))
        self.spbtn_offs_angle.set_value(self.prefs.getpref("ps_offs_angle", 0.0, float))

        self.halcomp.newpin("ps_offs_angle", hal.HAL_FLOAT, hal.HAL_OUT)
        self.halcomp.newpin("auto_rott", hal.HAL_BIT, hal.HAL_OUT)

        if self.chk_auto_rott.get_active():
            self.halcomp["auto_rott"] = True
            self.hal_led_auto_rott.hal_pin.set(1)
        self.halcomp["ps_offs_angle"] = self.spbtn_offs_angle.get_value()

    # --------------
    # Rotate Buttons
    # --------------
    def on_chk_auto_rott_toggled(self, gtkcheckbutton, data=None):
        self.halcomp["auto_rott"] = gtkcheckbutton.get_active()
        self.hal_led_auto_rott.hal_pin.set(gtkcheckbutton.get_active())
        self.prefs.putpref("chk_auto_rott", gtkcheckbutton.get_active(), bool)

    def on_btn_set_angle_released(self, gtkbutton, data=None):
        self.prefs.putpref("ps_offs_angle", self.spbtn_offs_angle.get_value(), float)

        self.display_result_a(self.spbtn_offs_angle.get_value())

        s = "G10 L2 P0"
        if self.halcomp["set_zero"]:
            s += " X%.4f" % self.halcomp["ps_offs_x"]
            s += " Y%.4f" % self.halcomp["ps_offs_y"]
        else:
            self.stat.poll()
            x = self.stat.position[0]
            y = self.stat.position[1]
            s += " X%.4f" % x
            s += " Y%.4f" % y
        s += " R%.4f" % self.spbtn_offs_angle.get_value()
        print("s=", s)
        self.gcode(s)
        self.vcp_reload()
        time.sleep(1)

    def on_spbtn_offs_angle_key_press_event(self, gtkspinbutton, data=None):
        self.on_common_spbtn_key_press_event("ps_offs_angle", gtkspinbutton, data)

    def on_spbtn_offs_angle_value_changed(self, gtkspinbutton, data=None):
        self.on_common_spbtn_value_changed("ps_offs_angle", gtkspinbutton, data)

    # Y+Y+
    @ProbeScreenBase.ensure_errors_dismissed
    def on_angle_yp_released(self, gtkbutton, data=None):
        a = self.current_position_with_offsets()
        s = self.move_rel_xy(0, -self.halcomp["ps_xy_clearance"])
        ycres = self.probe_y(direction = 1, move_enable = False, s = s)
        s = self.move_y(a[1] - self.halcomp["ps_xy_clearance"])
        s += self.move_rel_xy(self.halcomp["ps_edge_length"], 0)
        ypres = self.probe_y(direction = 1, move_enable = False, orient_enable = False, s = s)
        
        alfa = math.degrees(math.atan2(ypres - ycres, self.halcomp["ps_edge_length"]))
        
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "YcYpA",
            yc=ycres,
            yp=ypres,
            a=alfa,
        )
        
        # move XY to adj start point
        self.gcode(self.move_xy(a[0], ycres))
        
        self.rotate_coord_system(alfa)
            
    # Y-Y-
    @ProbeScreenBase.ensure_errors_dismissed
    def on_angle_ym_released(self, gtkbutton, data=None):
        a = self.current_position_with_offsets()
        s = self.move_rel_xy(0, self.halcomp["ps_xy_clearance"])
        ycres = self.probe_y(direction = -1, move_enable = False, s = s)
        s = self.move_y(a[1] + self.halcomp["ps_xy_clearance"])
        s += self.move_rel_xy(-self.halcomp["ps_edge_length"], 0)
        ymres = self.probe_y(direction = -1, move_enable = False, orient_enable = False, s = s)
        
        alfa = math.degrees(math.atan2(ycres - ymres, self.halcomp["ps_edge_length"]))
        
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "YmYcA",
            ym=ymres,
            yc=ycres,
            a=alfa,
        )
        
        # move XY to adj start point
        self.gcode(self.move_xy(a[0], ycres))
        
        self.rotate_coord_system(alfa)

    # X+X+
    @ProbeScreenBase.ensure_errors_dismissed
    def on_angle_xp_released(self, gtkbutton, data=None):
        self.probe_on()
        a = self.current_position_with_offsets()
        s = self.move_rel_xy(-self.halcomp["ps_xy_clearance"], 0)
        xcres = self.probe_x(direction = 1, move_enable = False, s = s)
        s = self.move_x(a[0] - self.halcomp["ps_xy_clearance"])
        s += self.move_rel_xy(0, -self.halcomp["ps_edge_length"])
        xpres = self.probe_x(direction = 1, move_enable = False, orient_enable = False, s = s)
        
        alfa = math.degrees(math.atan2(xcres - xpres, self.halcomp["ps_edge_length"]))
        
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XcXpA",
            xc=xcres,
            xp=xpres,
            a=alfa,
        )
        
        # move XY to adj start point
        self.gcode(self.move_xy(xcres, a[1]))
        
        self.rotate_coord_system(alfa)
            
    # X-X-
    @ProbeScreenBase.ensure_errors_dismissed
    def on_angle_xm_released(self, gtkbutton, data=None):
        a = self.current_position_with_offsets()
        s = self.move_rel_xy(self.halcomp["ps_xy_clearance"], 0)
        xcres = self.probe_x(direction = -1, move_enable = False, s = s)
        s = self.move_x(a[0] + self.halcomp["ps_xy_clearance"])
        s += self.move_rel_xy(0, self.halcomp["ps_edge_length"])
        xmres = self.probe_x(direction = -1, move_enable = False, orient_enable = False, s = s)
        
        alfa = math.degrees(math.atan2(xcres - xmres, self.halcomp["ps_edge_length"]))

        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XmXcA",
            xm=xmres,
            xc=xcres,
            a=alfa,
        )

        # move XY to adj start point
        self.gcode(self.move_xy(xcres, a[1]))
       
        self.rotate_coord_system(alfa)

    # --------------
    # Helper Methods
    # --------------
    def rotate_coord_system(self, a=0.0):
        self.spbtn_offs_angle.set_value(a)
        self.display_result_a(a)

        if self.chk_auto_rott.get_active():
            s = "G10 L2 P0"
            if self.halcomp["set_zero"]:
                s += " X%s" % self.halcomp["ps_offs_x"]
                s += " Y%s" % self.halcomp["ps_offs_y"]
            else:
                self.stat.poll()
                x = self.stat.position[0]
                y = self.stat.position[1]
                s += " X%s" % x
                s += " Y%s" % y
            s += " R%s" % a
            self.gcode(s)
            self.vcp_reload()
            time.sleep(1)
            
